from enum import IntEnum


class SHParticleType(IntEnum):
    """
    Particle list based on JPART from SHIELD-HIT12A, extended with ids 0 and -1.
    """
    all = -1
    unknown = 0
    neutron = 1
    proton = 2
    pi_minus = 3
    pi_plus = 4
    pi_zero = 5
    anti_neutron = 6
    anti_proton = 7
    kaon_minus = 8
    kaon_plus = 9
    kaon_zero = 10
    kaon_tilde = 11
    gamma = 12
    electron = 13
    positron = 14
    muon_minus = 15
    muon_plus = 16
    neutrino_e = 17
    anti_neutrino_e = 18
    neutrino_mu = 19
    anti_neutrino_mu = 20
    deuteron = 21
    triton = 22
    helium_3 = 23
    helium_4 = 24
    heavy_ion = 25

    def __str__(self):
        return self.name.replace('_', '-')


class SHHeavyIonType:
    particle_type = SHParticleType.heavy_ion

    def __init__(self):
        self.a = 0
        self.z = 0

    def __str__(self):
        result = str(self.particle_type)
        result += "_{:d}".format(self.z)
        result += "^{:d}".format(self.a)
        return result



import logging

logger = logging.getLogger(__name__)


class Inspector:
    def __init__(self, filename, options):
        logger.debug("Initialising Inspector writer")
        self.options = options

    def write(self, detector):
        """Print all keys and values from detector structure

        they include also a metadata read from binary output file
        """
        for name, value in sorted(detector.__dict__.items()):
            # be careful not to check for np.array but for np.ndarray!
            if name not in {'data', 'data_raw', 'error', 'error_raw', 'counter'}:  # skip non-metadata fields
                line = "{:24s}: '{:s}'".format(str(name), str(value))
                print(line)
        # print some data-related statistics
        print(75 * "*")
        print("Data min: {:g}, max: {:g}".format(detector.data.min(), detector.data.max()))

        if self.options.details:
            # print data scatter-plot if possible
            if detector.dimension == 1:
                try:
                    from hipsterplot import plot
                    print(75 * "*")
                    print("Data scatter plot")
                    plot(detector.data_raw)
                except ImportError:
                    logger.warning("Detailed summary requires installation of hipsterplot package")
            # print data histogram if possible
            try:
                from bashplotlib.histogram import plot_hist
                print(75 * "*")
                print("Data histogram")
                plot_hist(detector.data_raw, bincount=70, xlab=False, showSummary=True)
            except ImportError:
                logger.warning("Detailed summary requires installation of bashplotlib package")

        return 0



import sys
import argparse

from pymchelper.io import fromfile


def main(args=sys.argv[1:]):
    """
    Reading two files, one generated by Fluka and one generated by SHIELD-HIT12A
    :param args:
    :return:
    """

    # parsing command line options
    parser = argparse.ArgumentParser(args)
    parser.add_argument("--fluka", help='binary file generated with Fluka', type=str, required=True)
    parser.add_argument("--shield", help='binary file generated with SHIELD-HIT12A', type=str, required=True)
    parsed_args = parser.parse_args(args)

    # paths to binary files
    fluka_file_path = parsed_args.fluka
    sh12a_file_path = parsed_args.shield

    # creating empty Detector object and filling it with data read from Fluka file
    fluka_data = fromfile(fluka_file_path)

    # same as above but reading from SHIELD-HIT12A file
    sh12a_data = fromfile(sh12a_file_path)

    # printing some output on the screen
    print("Fluka bins in X: {:d}, Y: {:d}, Z: {:d}".format(fluka_data.x.n, fluka_data.y.n, fluka_data.z.n))
    print("First bin of fluka data", fluka_data.data[0, 0, 0])

    print("SHIELD-HIT12A bins in X: {:d}, Y: {:d}, Z: {:d}".format(sh12a_data.x.n, sh12a_data.y.n, sh12a_data.z.n))
    print("First bin of SHIELD-HIT12A data", sh12a_data.data[0, 0, 0])

    # comparing file contents
    print("Difference Fluka - SHIELD-HIT12A", (sh12a_data.data_raw - fluka_data.data_raw)[0:5], "...")


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



# $Id: log.py 3456 2015-03-04 10:42:14Z bnv $
#
# Copyright and User License
# ~~~~~~~~~~~~~~~~~~~~~~~~~~
# Copyright Vasilis.Vlachoudis@cern.ch for the
# European Organization for Nuclear Research (CERN)
#
# All rights not expressly granted under this license are reserved.
#
# Installation, use, reproduction, display of the
# software ("flair"), in source and binary forms, are
# permitted free of charge on a non-exclusive basis for
# internal scientific, non-commercial and non-weapon-related
# use by non-profit organizations only.
#
# For commercial use of the software, please contact the main
# author Vasilis.Vlachoudis@cern.ch for further information.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following
# conditions are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
#
# DISCLAIMER
# ~~~~~~~~~~
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
# NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY, OF
# SATISFACTORY QUALITY, AND FITNESS FOR A PARTICULAR PURPOSE
# OR USE ARE DISCLAIMED. THE COPYRIGHT HOLDERS AND THE
# AUTHORS MAKE NO REPRESENTATION THAT THE SOFTWARE AND
# MODIFICATIONS THEREOF, WILL NOT INFRINGE ANY PATENT,
# COPYRIGHT, TRADE SECRET OR OTHER PROPRIETARY RIGHT.
#
# LIMITATION OF LIABILITY
# ~~~~~~~~~~~~~~~~~~~~~~~
# THE COPYRIGHT HOLDERS AND THE AUTHORS SHALL HAVE NO
# LIABILITY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL,
# CONSEQUENTIAL, EXEMPLARY, OR PUNITIVE DAMAGES OF ANY
# CHARACTER INCLUDING, WITHOUT LIMITATION, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES, LOSS OF USE, DATA OR PROFITS,
# OR BUSINESS INTERRUPTION, HOWEVER CAUSED AND ON ANY THEORY
# OF CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE), PRODUCT
# LIABILITY OR OTHERWISE, ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGES.

import logging

__author__ = "Vasilis Vlachoudis"
__email__ = "Vasilis.Vlachoudis@cern.ch"

# -------------------------------------------------------------------------------
_log = None

logger = logging.getLogger(__name__)


def set_log(l):
    global _log
    _log = l


def say(*kw):
    txt = " ".join(map(str, kw))
    if _log:
        _log(txt)
    else:
        logger.info(txt)


def null():
    pass



"""
Tool for creating MC input files using user-specified tables and ranges.

2019 - Niels Bassler
"""

import os
import sys
import errno
import logging
import argparse

import numpy as np

logger = logging.getLogger(__name__)


class Config():
    """
    Reading the config file.
    """
    def __init__(self, fn):
        with open(fn) as _f:
            self.lines = _f.readlines()
            self.base_dir = os.path.dirname(fn)

            # script must run relative to location of config file.
            # this is needed, otherwise symlink creation will fail, as these are relative
            # and expect that the target exists.
            if self.base_dir:
                os.chdir(self.base_dir)
            self.parse()

    def parse(self):
        """
        Parse configuration file.
        All data are read into dicts. There are two dicts: a constant and variable (tabluated) one.
        """

        self.const_dict = {}  # list of contant assignments
        self.table_dict = {}  # list of table assignments

        keys = []
        vals = []

        _first_line_in_table = True

        for line in self.lines:
            # skip comments
            if line.startswith('#'):
                continue

            # constant assigments
            if "=" in line:
                _v = line.split("=")
                self.const_dict[_v[0].strip()] = _v[1].strip()
                continue

            # check if we are starting a table
            if len(line.split()) > 0:
                if _first_line_in_table:
                    keys = line.split()
                    _first_line_in_table = False
                else:
                    vals.append(line.split())

        # after parsing all lines setup the variable table, if it exists:
        if keys and vals:
            for i, key in enumerate(keys):
                self.table_dict[key] = [val[i] for val in vals]

        _files = []
        for item in self.const_dict["FILES"].split(","):
            _files.append(item.strip())
        self.const_dict["FILES"] = _files

        _files = []
        for item in self.const_dict["SYMLINKS"].split(","):
            _files.append(item.strip())
        self.const_dict["SYMLINKS"] = _files


class McFile():
    """
    General MC single file object.
    This will be used for the template files as well as the generated output files.
    """
    def __init__(self):
        self.fname = ""  # filename
        self.path = ""   # full path to this file (may be relative)
        self.lines = []  # list of lines inside this file
        self.symlink = False  # marker if file is a symlink
        self.templ_dir = ""  # template directory

    def write(self):
        """
        Write self to disk, create symlink if that is the case.
        """
        # check if target directory exists, create it, if not.
        try:
            os.makedirs(os.path.dirname(self.path))
        except OSError as e:  # when python 2.7 is dropped this can be replaces tih FileExistsError: pass
            if e.errno == errno.EEXIST:
                pass  # silently accept existing directories
            else:
                logger.error('Directory not created.')
                raise

        if self.symlink:
            link_file = os.path.join(self.templ_dir, self.fname)
            link_target = os.path.join(self.path)
            link_name = os.path.relpath(link_file, os.path.dirname(link_target))

            try:
                os.symlink(link_name, link_target)
            except AttributeError as e:  # python 2.7 on Windows cannot create symlinks
                logger.error('Symlink not created.')
                print("Cannot create a link to the file, please check if you are using Linux or Python 3.")
                raise e
            except OSError as e:  # when python 2.7 is dropped this can be replaces tih FileExistsError: pass
                if e.errno == errno.EEXIST:
                    pass  # silently accept existing symlinks
                else:
                    logger.error('Symlink not created.')
                    raise
        else:
            with open(self.path, 'w') as _f:
                _f.writelines(self.lines)


class Template():
    """
    Read all files and symlinks specified in the config file, and place them in a list of McFile objects.
    """
    def __init__(self, cfg):
        self.files = []
        self.read(cfg)

    def read(self, cfg):
        """
        Reads all template files, and creates a list of McFile objects in self.files.
        """

        fname_list = cfg.const_dict["FILES"] + cfg.const_dict["SYMLINKS"]

        for fname in fname_list:
            tf = McFile()
            tf.fname = fname
            tf.path = os.path.join(cfg.const_dict["TDIR"], fname)
            tf.templ_dir = cfg.const_dict["TDIR"]

            with open(tf.path) as _f:
                tf.lines = _f.readlines()

            if fname in cfg.const_dict["SYMLINKS"]:
                tf.symlink = True
            else:
                tf.symlink = False
            self.files.append(tf)


class Generator():
    """
    This generates and writes the output files based on the loaded template files and the config file.
    """
    def __init__(self, templ, cfg):
        """
        Logic attached to the various keys is in here.
        templ is a Template object and cfg is a Config object.
        """
        # create a new dict, with all keys, but single unique values only:
        # this is the "current unique dictionary"
        u_dict = cfg.const_dict.copy()

        # loop_keys are special keys which cover a numerical range in discrete steps.
        # here we will identify them, and for each loop_key, there will be a range setup.
        # loop_keys are identified by the "_MIN" suffix:
        loop_keys = []
        for key in cfg.table_dict.keys():
            if "_MIN" in key:
                loop_keys.append(key.strip("_MIN") + "_")

        # reuse any key from table to calculate the length of the table
        # this means that the table must be homogenous, i.e. every key must have the same amount of values.
        _vals = cfg.table_dict[key]
        for i, val in enumerate(_vals):
            for key in cfg.table_dict.keys():
                u_dict[key] = cfg.table_dict[key][i]  # only copy the ith value

            # now prepare the ranges for every loop_key
            for loop_key in loop_keys:
                _lmin = float(cfg.table_dict[loop_key + "MIN"][i])
                _lmax = float(cfg.table_dict[loop_key + "MAX"][i])
                _lst = float(cfg.table_dict[loop_key + "STEP"][i])
                loop_vals = np.arange(_lmin, _lmax, _lst)
                for loop_val in loop_vals:
                    u_dict[loop_key] = loop_val

                    # set the relative energy spread:
                    if "E_" in u_dict and "DE_FACTOR" in u_dict:
                        _de = float(u_dict["E_"]) * float(u_dict["DE_FACTOR"])
                        u_dict["DE_"] = "{:.3f}".format(_de)  # HARDCODED float format for DE_

                    # at this point, the dict is fully set.
                    self.write(templ, u_dict)

    @staticmethod
    def get_keys(s):  # This is currently not used, but kept for future use.
        """
        return list of ${} keys in string
        """
        r = []

        if "${" in s:
            _i = [i for i, d in enumerate(s) if d == "{"]
            _ii = [i for i, d in enumerate(s) if d == "}"]

            for i in zip(_i, _ii):
                r.append(s[i[0] + 1:i[1]])
        return r

    @staticmethod
    def lreplace(s, f, r):
        """
        Left adjusted replacement of string f with string r, in string s.

        This function is implemented in order to fill in data in FORTRAN77 fields,
        which are tied to certain positions on the line, i.e. subsequent values
        may not be shifted.

        Finds f in s and replaces it with r, but left adjusted, retaining line length.
        If r is shorter than f, remaining chars will be space padded.
        If r is larger than f, then characters will be overwritten.
        A copy of s with the replacement is returned.
        """
        if f in s:
            _idx = s.find(f)
            if len(r) < len(f):
                _r = r + " " * (len(f) - len(r))
            else:
                _r = r
            text = s[:_idx] + _r + s[_idx + len(_r):]
            return text

    def write(self, t, u_dict):
        """
        Write a copy of the template, using the substitutions as specifed in
        the unique dictionary u_dict.

        "Unique", means that any _MIN _MAX _STEP type variables have been set.
        """

        _wd = u_dict["WDIR"]

        # check if any keys are in WDIR subsitutions
        for key in u_dict.keys():
            token = "${" + key + "}"
            if token in _wd:

                _s = u_dict[key]
                if isinstance(_s, float):
                    _s = "{:08.3f}".format(u_dict[key])  # HARDCODED float format for directory string
                _wd = _wd.replace(token, _s)
        work_dir = _wd

        for tf in t.files:  # tf = template filename
            of = McFile()  # of = output file object
            of.fname = tf.fname
            of.path = os.path.join(work_dir, tf.fname)
            of.templ_dir = tf.templ_dir

            # symlinks should not be parsed for tokens.
            if tf.symlink:
                of.symlink = True
            else:
                of.symlink = False

                for line in tf.lines:
                    for key in u_dict.keys():
                        token = "${" + key + "}"
                        while token in line:
                            _s = u_dict[key]
                            if isinstance(_s, float):
                                _s = "{:.3f}".format(u_dict[key])  # HARDCODED float format for loop_keys
                            line = self.lreplace(line, token, _s)
                    of.lines.append(line)
            # end loop over t.files
            of.write()


def main(args=sys.argv[1:]):
    """
    Main function.
    """

    import pymchelper
    parser = argparse.ArgumentParser()
    parser.add_argument('fconf', metavar="config.txt", type=argparse.FileType('r'),
                        help="path to config file.",
                        default=sys.stdin)
    parser.add_argument('-v', '--verbosity', action='count', help="increase output verbosity", default=0)
    parser.add_argument('-V', '--version', action='version', version=pymchelper.__version__)
    args = parser.parse_args(args)

    if args.verbosity == 1:
        logging.basicConfig(level=logging.INFO)
    if args.verbosity > 1:
        logging.basicConfig(level=logging.DEBUG)

    cfg = Config(args.fconf.name)
    t = Template(cfg)

    Generator(t, cfg)


if __name__ == '__main__':
    logging.basicConfig()
    sys.exit(main(sys.argv[1:]))



"""
Reads PLD file in IBA format and convert to sobp.dat
which is readable by FLUKA with source_sampler.f and by SHIELD-HIT12A.

TODO: Translate energy to spotsize.
"""
import sys
import logging
import argparse
from math import exp, log

logger = logging.getLogger(__name__)


def dedx_air(energy):
    """
    Calculate the mass stopping power of protons in air following ICRU 49.
    Valid from 1 to 500 MeV only.

    :params energy: Proton energy in MeV
    :returns: mass stopping power in MeV cm2/g
    """
    if energy > 500.0 or energy < 1.0:
        logger.error("Proton energy must be between 1 and 500 MeV.")
        raise ValueError("Energy = {:.2f} out of bounds.".format(energy))

    x = log(energy)
    y = 5.4041 - 0.66877 * x - 0.034441 * (x**2) - 0.0010707 * (x**3) + 0.00082584 * (x**4)
    return exp(y)


class Layer(object):
    """
    Class for handling Layers.
    """
    def __init__(self, spotsize, energy, meterset, elsum, repaints, elements):
        self.spotsize = float(spotsize)
        self.energy = float(energy)
        self.meterset = float(meterset)   # MU sum of this + all previous layers
        self.elsum = float(elsum)         # sum of elements in this layer
        self.repaints = int(repaints)     # number of repaints
        self.elements = elements          # number of elements
        self.spots = int(len(elements) / 2)  # there are two elements per spot

        self.x = [0.0] * self.spots
        self.y = [0.0] * self.spots
        self.w = [0.0] * self.spots       # MU weight
        self.rf = [0.0] * self.spots      # fluence weight

        j = 0
        for element in elements:
            token = element.split(",")
            if token[3] != "0.0":
                self.x[j] = float(token[1].strip())
                self.y[j] = float(token[2].strip())
                self.w[j] = float(token[3].strip())  # meterset weight of this spot
                self.rf[j] = self.w[j]
                j += 1  # only every second token has a element we need.


class PLDRead(object):
    """
    Class for handling PLD files.
    """

    def __init__(self, fpld):
        """ Read the rst file."""

        pldlines = fpld.readlines()
        pldlen = len(pldlines)
        logger.info("Read {} lines of data.".format(pldlen))

        self.layers = []

        # parse first line
        token = pldlines[0].split(",")
        self.beam = token[0].strip()
        self.patient_iD = token[1].strip()
        self.patient_name = token[2].strip()
        self.patient_initals = token[3].strip()
        self.patient_firstname = token[4].strip()
        self.plan_label = token[5].strip()
        self.beam_name = token[6].strip()
        self.mu = float(token[7].strip())
        self.csetweight = float(token[8].strip())
        self.nrlayers = int(token[9].strip())  # number of layers

        for i in range(1, pldlen):  # loop over all lines starting from the second one
            line = pldlines[i]
            if "Layer" in line:
                header = line

                token = header.split(",")
                # extract the subsequent lines with elements
                el_first = i + 1
                el_last = el_first + int(token[4])

                elements = pldlines[el_first:el_last]

                # read number of repaints only if 5th column is present, otherwise set to 0
                repaints_no = 0
                if 5 in token:
                    repaints_no = token[5].strip()

                self.layers.append(Layer(token[1].strip(),  # Spot1
                                         token[2].strip(),  # energy
                                         token[3].strip(),  # cumulative meter set weight including this layer
                                         token[4].strip(),  # number of elements in this layer
                                         repaints_no,  # number of repaints.
                                         elements))  # array holding all elements for this layer


def main(args=sys.argv[1:]):
    """ Main function of the pld2sobp script.
    """

    import pymchelper

    # _scaling holds the number of particles * dE/dx / MU = some constant
    # _scaling = 8.106687e7  # Calculated Nov. 2016 from Brita's 32 Gy plan. (no dE/dx)
    _scaling = 5.1821e8  # Estimated calculation Apr. 2017 from Brita's 32 Gy plan.

    parser = argparse.ArgumentParser()
    parser.add_argument('fin', metavar="input_file.pld", type=argparse.FileType('r'),
                        help="path to .pld input file in IBA format.",
                        default=sys.stdin)
    parser.add_argument('fout', nargs='?', metavar="output_file.dat", type=argparse.FileType('w'),
                        help="path to the SHIELD-HIT12A/FLUKA output file, or print to stdout if not given.",
                        default=sys.stdout)
    parser.add_argument('-f', '--flip', action='store_true', help="flip XY axis", dest="flip", default=False)
    parser.add_argument('-d', '--diag', action='store_true', help="prints additional diagnostics",
                        dest="diag", default=False)
    parser.add_argument('-s', '--scale', type=float, dest='scale',
                        help="number of particles*dE/dx per MU.", default=_scaling)
    parser.add_argument('-v', '--verbosity', action='count', help="increase output verbosity", default=0)
    parser.add_argument('-V', '--version', action='version', version=pymchelper.__version__)
    args = parser.parse_args(args)

    if args.verbosity == 1:
        logging.basicConfig(level=logging.INFO)
    if args.verbosity > 1:
        logging.basicConfig(level=logging.DEBUG)

    pld_data = PLDRead(args.fin)
    args.fin.close()

    # SH12A takes any form of list of values, as long as the line is shorter than 78 Chars.
    outstr = "{:-10.6f} {:-10.2f} {:-10.2f} {:-10.2f} {:-16.6E}\n"

    meterset_weight_sum = 0.0
    particles_sum = 0.0

    for layer in pld_data.layers:
        spotsize = 2.354820045 * layer.spotsize * 0.1  # 1 sigma im mm -> 1 cm FWHM

        for spot_x, spot_y, spot_w, spot_rf in zip(layer.x, layer.y, layer.w, layer.rf):

            weight = spot_rf * pld_data.mu / pld_data.csetweight
            # Need to convert to weight by fluence, rather than weight by dose
            # for building the SOBP. Monitor Units (MU) = "meterset", are per dose
            # in the monitoring Ionization chamber, which returns some signal
            # proportional to dose to air. D = phi * S => MU = phi * S(air)
            phi_weight = weight / dedx_air(layer.energy)

            # add number of paricles in this spot
            particles_spot = args.scale * phi_weight
            particles_sum += particles_spot

            meterset_weight_sum += spot_w

            layer_xy = [spot_x * 0.1, spot_y * 0.1]

            if args.flip:
                layer_xy.reverse()

            args.fout.writelines(outstr.format(layer.energy * 0.001,  # MeV -> GeV
                                               layer_xy[0],
                                               layer_xy[1],
                                               spotsize,
                                               particles_spot))

    logger.info("Data were scaled with a factor of {:e} particles*S/MU.".format(args.scale))
    if args.flip:
        logger.info("Output file was XY flipped.")

    if args.diag:
        energy_list = [layer.energy for layer in pld_data.layers]

        # double loop over all layers and over all spots in a layer
        spot_x_list = [x for layer in pld_data.layers for x in layer.x]
        spot_y_list = [y for layer in pld_data.layers for y in layer.y]
        spot_w_list = [w for layer in pld_data.layers for w in layer.w]

        print("")
        print("Diagnostics:")
        print("------------------------------------------------")
        print("Total MUs              : {:10.4f}".format(pld_data.mu))
        print("Total meterset weigths : {:10.4f}".format(meterset_weight_sum))
        print("Total particles        : {:10.4e} (estimated)".format(particles_sum))
        print("------------------------------------------------")
        for i, energy in enumerate(energy_list):
            print("Energy in layer {:3}    : {:10.4f} MeV".format(i, energy))
        print("------------------------------------------------")
        print("Highest energy         : {:10.4f} MeV".format(max(energy_list)))
        print("Lowest energy          : {:10.4f} MeV".format(min(energy_list)))
        print("------------------------------------------------")
        print("Spot X         min/max : {:10.4f} {:10.4f} mm".format(min(spot_x_list), max(spot_x_list)))
        print("Spot Y         min/max : {:10.4f} {:10.4f} mm".format(min(spot_y_list), max(spot_y_list)))
        print("Spot meterset  min/max : {:10.4f} {:10.4f}   ".format(min(spot_w_list), max(spot_w_list)))
        print("")
    args.fout.close()


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



import os
import subprocess


def git_version():
    """
    Inspired by https://github.com/numpy/numpy/blob/master/setup.py
    :return: the git revision as a string
    """
    def _minimal_ext_cmd(cmd):
        # construct minimal environment
        env = {}
        for k in ['SYSTEMROOT', 'PATH', 'HOME']:
            v = os.environ.get(k)
            if v is not None:
                env[k] = v
        # LANGUAGE is used on win32
        env['LANGUAGE'] = 'C'
        env['LANG'] = 'C'
        env['LC_ALL'] = 'C'
        out = subprocess.Popen(cmd, stdout=subprocess.PIPE, env=env).communicate()[0]
        return out

    try:
        out = _minimal_ext_cmd(['git', 'describe', '--tags', '--long'])
        GIT_REVISION = out.strip().decode('ascii')
        if GIT_REVISION:
            no_of_commits_since_last_tag = int(GIT_REVISION.split('-')[1])
            tag_name = GIT_REVISION.split('-')[0][1:]
            if no_of_commits_since_last_tag == 0:
                version = tag_name
            else:
                version = '{}+rev{}'.format(tag_name, no_of_commits_since_last_tag)
        else:
            version = "Unknown"
    except OSError:
        version = "Unknown"

    return version



import logging

import numpy as np

logger = logging.getLogger(__name__)


class ExcelWriter:
    """
    Supports writing XLS files (MS Excel 2003 format)
    """
    def __init__(self, filename, options):
        self.filename = filename
        if not self.filename.endswith(".xls"):
            self.filename += ".xls"

    def write(self, detector):

        try:
            import xlwt
        except ImportError as e:
            logger.error("Generating Excel files not available on your platform (you are probably running Python 3.2).")
            raise e

        # save only 1-D data
        if detector.dimension != 1:
            logger.warning("Detector dimension {:d} != 1, XLS output not supported".format(detector.dimension))
            return 1

        # create workbook with single sheet
        wb = xlwt.Workbook()
        ws = wb.add_sheet('Data')

        # save X axis data
        for i, x in enumerate(detector.plot_axis(0).data):
            ws.write(i, 0, x)

        # save Y axis data
        for i, y in enumerate(detector.data_raw):
            ws.write(i, 1, y)

        # save error column (if present)
        if np.all(np.isfinite(detector.error_raw)):
            for i, e in enumerate(detector.error_raw):
                ws.write(i, 2, e)

        # save file
        wb.save(self.filename)

        return 0
